<?php
/**
 * @category   Shipping
 * @package    Eniture_ResidentialAddressDetection
 * @author     Eniture Technology : <sales@eniture.com>
 * @website    http://eniture.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
/**
 * class that instantiate guide section html block template
 */
namespace Eniture\ResidentialAddressDetection\Helper;
use \Magento\Framework\App\Helper\AbstractHelper;

class Data extends AbstractHelper {
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var \Magento\Config\Model\ResourceModel\Config
     */
    private $resourceConfig;
    /**
     * @var string
     */
    private $planStatusReqUrl = 'https://eniture.com/ws/addon/smartStreetApi/index.php';
    /**
     * @var \Magento\Framework\App\Cache\TypeListInterface
     */
    private $cacheTypeList;
    /**
     * @var \Magento\Framework\App\Cache\Frontend\Pool
     */
    private $cacheFrontendPool;
    /**
     * @var enitureModuleLicenseKey
     */
    private $enitureModuleLicenseKey;
    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    private $request;
    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    private $curl;
    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\Framework\App\Cache\Frontend\Pool $cacheFrontendPool
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Framework\App\RequestInterface $httpRequest
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     */
    public function __construct(
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Framework\App\ResourceConnection $resource,
            \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
            \Magento\Framework\App\Cache\Frontend\Pool $cacheFrontendPool,
            \Magento\Config\Model\ResourceModel\Config $resourceConfig,
            \Magento\Framework\App\RequestInterface $httpRequest,
            \Magento\Framework\HTTP\Client\Curl $curl
            ) {
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
        $this->scopeConfig = $context->getScopeConfig();
        $this->resourceConfig = $resourceConfig;
        $this->request = $httpRequest;
        $this->curl = $curl;
        parent::__construct($context);
    }

    /**
     * @param $licenseKey
     * @return array
     */
    public function resAddDetectDataHandling($licenseKey)
    {
        $this->enitureModuleLicenseKey  = $licenseKey;
        $data = $this->getCurrentPlanStatus();
        return $data;
    }
    
    /**
     * getCurrentPlanStatus
     * @return array
     */
    public function getCurrentPlanStatus() {
        $trialStart = $this->scopeConfig->getValue("enitureRAD/trial/use");
        if($trialStart == '0'){
            $trialActive = $this->getRequestArray('c', 'TR');
            if(isset($trialActive->severity) && $trialActive->severity == 'SUCCESS'){
                $this->resourceConfig->saveConfig(
                    'binBox/trial/use',
                    '1',
                    \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT, 
                    \Magento\Store\Model\Store::DEFAULT_STORE_ID
                );
            }
        }
        
        $planStatus = $this->getRequestArray('s', '');
        return $this->setPlanStatusData($planStatus);
    }

    /**
     * @return string
     */
    public function checkSmallModuleTrial(){
        $smallTrialMsg = '';
        $smallModuleStatus = $this->getRequestArray('pluginType', '');
        if(isset($smallModuleStatus->severity) && $smallModuleStatus->severity == 'SUCCESS' && $smallModuleStatus->pluginType == 'trial'){
            $smallTrialMsg = 'The Small Package Quotes module must have active paid license to continue to use this feature.';
        }
        return $smallTrialMsg;
    }

    /**
     * @param $statusResponse
     * @return array
     */
    public function setPlanStatusData($statusResponse) {
        if(isset($statusResponse->severity) && $statusResponse->severity == 'SUCCESS'){
            $this->showStatus = 'yes';
            $this->statusMsg = 'SUCCESS';
            $planStatus = isset($statusResponse->status)?$statusResponse->status:'';
            
            if($planStatus){
                
                $this->packageHits          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageHits))) ? $planStatus->subscribedPackage->packageHits : "NA";
                $this->packageDuration      = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageDuration))) ? $planStatus->subscribedPackage->packageDuration : "NA";
                $this->packageCost          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageCost))) ? "($" .$planStatus->subscribedPackage->packageCost. ")": "NA";
                $this->subscriptionTime     = (isset($planStatus->subscriptionInfo) && (!empty($planStatus->subscriptionInfo->subscriptionTime))) ? "Start date: " . $this->formateDateTime($planStatus->subscriptionInfo->subscriptionTime) : "NA";
                $this->expiryTime           = (isset($planStatus->subscriptionInfo) && (!empty($planStatus->subscriptionInfo->expiryTime))) ? "End date: " . $this->formateDateTime($planStatus->subscriptionInfo->expiryTime) : "NA";
                $this->consumedHits         = (isset($planStatus->subscribedPackageHitsStatus) && (!empty($planStatus->subscribedPackageHitsStatus->consumedHits))) ? $planStatus->subscribedPackageHitsStatus->consumedHits : "0";
                $this->consumedHitsPrcent   = (isset($planStatus->subscribedPackageHitsStatus) && (!empty($planStatus->subscribedPackageHitsStatus->consumedHitsPrcent))) ? $planStatus->subscribedPackageHitsStatus->consumedHitsPrcent. "%" : "NA";
                $this->statusRequestTime    = "(". $statusResponse->statusRequestTime .")";
            
                $this->packageSCAC          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageSCAC))) ? $planStatus->subscribedPackage->packageSCAC: "NA";
                $this->checkTrail           = (isset($planStatus->subscribedPackage)? $planStatus->subscribedPackage->subscribedPackageName : "NA" );
            
                $this->nextSubcribedPackage       = (isset($planStatus->nextSubcribedPackage) && (!empty($planStatus->nextSubcribedPackage->nextSubscriptionSCAC))) ? $planStatus->nextSubcribedPackage->nextSubscriptionSCAC: "NA";
                $this->nextSubcribedPackagestatus = (isset($planStatus->nextSubcribedPackage) && (!empty($planStatus->nextSubcribedPackage->nextToBeChargedStatus))) ? $planStatus->nextSubcribedPackage->nextToBeChargedStatus: "NA";
                
                $this->subscriptionStatus = (isset($planStatus->subscriptionInfo) && $planStatus->subscriptionInfo->subscriptionStatus == '1')?'active':'deactive';
            }   
        }else{
            $this->showStatus = 'no';
            $this->statusMsg = isset($statusResponse->Message) ? $statusResponse->Message : 'Your current subscription is expired.';
        }
        $this->ListOfPackages       = isset($statusResponse->ListOfPackages) ? $statusResponse->ListOfPackages->Info : '';
        return $this->setPlanDataArr();
    }

    /**
     * @return array
     */
    function setPlanDataArr() {
        $arr = array(
            'showStatus'        => $this->showStatus,
            'statusMsg'         => $this->statusMsg,
            'packageHits'       => (isset($this->packageHits))?$this->packageHits:'NA',
            'packageDuration'   => (isset($this->packageDuration))?$this->packageDuration:'NA',
            'packageCost'       => (isset($this->packageCost))?$this->packageCost:'NA',
            'subscriptionTime'  => (isset($this->subscriptionTime))?$this->subscriptionTime:'NA',
            'expiryTime'        => (isset($this->expiryTime))?$this->expiryTime:'NA',
            'consumedHits'      => (isset($this->consumedHits))?$this->consumedHits:'NA',
            'consumedHitsPrcent' => (isset($this->consumedHitsPrcent))?$this->consumedHitsPrcent:'NA',
            'statusRequestTime' => (isset($this->statusRequestTime))?$this->statusRequestTime:'NA',
            'packageSCAC'       => (isset($this->packageSCAC))?$this->packageSCAC:'NA',
            'ListOfPackages'    => $this->ListOfPackages,
            'checkTrail'        => (isset($this->checkTrail)? $this->checkTrail : "NA" ),
            'nextSubcribedPackage'        => (isset($this->nextSubcribedPackage)? $this->nextSubcribedPackage : "NA" ),
            'nextSubcribedPackagestatus'  => (isset($this->nextSubcribedPackagestatus)? $this->nextSubcribedPackagestatus : "NA" ),
            'subscriptionStatus'  => (isset($this->subscriptionStatus)? $this->subscriptionStatus : "NA" )
        );

        return $arr;
    }

    /**
     * @param $datetime
     * @return false|string
     */
    public function formateDateTime($datetime) {

        $date = date_create($datetime);
        return date_format($date, "M. d, Y");
    }

    /**
     * @return mixed
     */
    public function radUseSuspended()
    {
        $radSuspend     = $this->scopeConfig->getValue("resaddressdetection/suspend/value");
        return $radSuspend;
    }

    /**
     * @param $action
     * @param $package
     * @return mixed
     */
    public function getRequestArray($action, $package)
    {
        $postArr = array(
            'platform'      => 'magento2',
            'request_key'   => md5(microtime() . rand()),
            'action'        => $action,
            'package'       => $package,
            'serverName'   => $this->request->getServer('SERVER_NAME'),
            'licenseKey'   => $this->enitureModuleLicenseKey,
        );
        
        return $this->sendRequest($this->planStatusReqUrl,$postArr);
    }
    
    /**
     * clear magento cache
     */
    function cacheClean()
    {
        $types = array('config','layout','block_html','collections','reflection','db_ddl','eav','config_integration','config_integration_api','full_page','translate','config_webservice');
        foreach ($types as $type) {
            $this->cacheTypeList->cleanType($type);
        }
        foreach ($this->cacheFrontendPool as $cacheFrontend) {
            $cacheFrontend->getBackend()->clean();
        }
    }
    
    /**
     * This function send request and return responce
     * $isAssocArray Paramiter When TRUE, then returned objects will 
     * be converted into associative arrays, otherwise its an object
     * @param $url
     * @param $postData
     * @param $isAssocArray
     * @return
     */
    public function sendRequest($url, $postData, $isAssocArray=FALSE)
    {
        $fieldString = http_build_query($postData);
        $this->curl->post($url, $fieldString);
        $output = $this->curl->getBody();
        $result = json_decode($output, $isAssocArray);
        return $result;
    }
}